#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <stdio.h>



#include "sht75.h"



#ifndef F_CPU
/* In neueren Version der WinAVR/Mfile Makefile-Vorlage kann
   F_CPU im Makefile definiert werden, eine nochmalige Definition
   hier wuerde zu einer Compilerwarnung fuehren. Daher "Schutz" durch
   #ifndef/#endif 
 
   Dieser "Schutz" kann zu Debugsessions fhren, wenn AVRStudio 
   verwendet wird und dort eine andere, nicht zur Hardware passende 
   Taktrate eingestellt ist: Dann wird die folgende Definition 
   nicht verwendet, sondern stattdessen der Defaultwert (8 MHz?) 
   von AVRStudio - daher Ausgabe einer Warnung falls F_CPU
   noch nicht definiert: */
#warning "F_CPU war noch nicht definiert, wird nun nachgeholt mit 4000000"
#define F_CPU 8000000UL  // Systemtakt in Hz - Definition als unsigned long beachten 
                         // Ohne ergeben sich unten Fehler in der Berechnung
#endif

#define BAUD 9600UL      // Baudrate
 
// Berechnungen
#define UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)   // clever runden
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))     // Reale Baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/BAUD) // Fehler in Promille, 1000 = kein Fehler.
 
#if ((BAUD_ERROR<990) || (BAUD_ERROR>1010))
  #error Systematischer Fehler der Baudrate grsser 1% und damit zu hoch! 
#endif


#define MSG_size 34				// Davon 2byte fr Status (MSG Nr, MSG Lnge)
#define Anzahl_Meldungen 240	// Anzahl der Meldungen


unsigned char rep;
static unsigned char index;
static unsigned char inbuf[50];

unsigned char error=0;
unsigned char checksum;

char disp_buffer[80]; 



//****************************************************************************
void answer_uart(unsigned char c){
while (!(UCSRA & (1<<UDRE)))  /* warten bis Senden moeglich                */
    {
    }
 
    UDR = c;
}

//****************************************************************************
int uart_putc(unsigned char c)
{
	while (!(UCSRA & (1<<UDRE)))
	{
	}
	
	UDR = c;
	return 0;
}

//****************************************************************************
void uart_puts (char *s)
{
	while (*s)
	{
		uart_putc(*s);
		s++;
	}
}





	
//----------------------------------------------------------------------------
ISR(USART_RXC_vect)
{
	 unsigned int c;
	 c = UDR;


	 
	 if ((c==0xFF) & (inbuf[0]==0xAA) & (inbuf[1]== 0xAA))
	 {
		answer_uart(0x15);
		index = 0;
		cli();
		wdt_enable(0);
		while(1) {};
	 }
	 
	 
	 else 
	 {	
		inbuf[index] = c;
		index++;
	 }

}

//----------------------------------------------------------------------------
// USART Init
void init_uart(void) 
{
	UBRRL = UBRR_VAL;
	UCSRB = UCSRB | (1<<RXEN) | (1<<TXEN) | (1<<RXCIE);
//	UCSRB = UCSRB | (1<<TXEN);
	UCSRC = (1<<URSEL) | (1<<UCSZ1) | (1<<UCSZ0);

}


int main(void)			// Mainprogramm
{


	// Disable Watchdog. Wird vom Bootloader bentigt.
	MCUCSR = 0;
	wdt_disable();
	
	
	init_uart();
	
	
	
	
	sht75_array sht75;
	
	uint8_t sht75_status;
	uint8_t sht75_error;
	
	ShtInit();
	sei();

	answer_uart(0x06);
	
	_delay_ms(1000);
	
	ShtReset();
	ShtReadEverything(&sht75);
	sht75_error = ShtReadStatus(&sht75_status);

	answer_uart(sht75_error);
	
	
	if (sht75_error == 0)
	{
		sprintf(disp_buffer,"Temp: %+02.2f Feuchte: %2.2f Taupunkt: %+02.2f Status: %02x Error: %02x\n\r",sht75.Temperature,sht75.Humidity,sht75.Dewpoint,sht75_status,sht75_error);
		uart_puts(disp_buffer);
	}
	
	while(1) {
	
	
	


	}

	
}